import dhl_delivery.quote
import dhl_delivery.pickup
import dhl_delivery.shipment

dhl_site_id = ''
dhl_site_password = ''
dhl_account_no = ''

dhl_response_flag = False
dhl_xml_flag = False
dhl_delivery_label_path = ''

dhl_delivery.dhl_site_id = dhl_site_id
dhl_delivery.dhl_site_password = dhl_site_password
dhl_delivery.dhl_account_no = dhl_account_no

dhl_delivery.dhl_delivery_label_path = dhl_delivery_label_path
dhl_delivery.dhl_xml_flag = dhl_xml_flag
dhl_delivery.dhl_response_flag = dhl_xml_flag
dhl_delivery.max_response_time = 60

# for delivery
dhl_delivery.delivery_payment_type = 'S'  # S -  always paid by shipper
dhl_delivery.duty_payment_type = 'R'  # duty always will be paid by Reciver

quote = dhl_delivery.quote.Quote()
pickup = dhl_delivery.pickup.Pickup()
shipment = dhl_delivery.shipment.Shipment()



import dhl_delivery
import dhl_delivery.config
import dhl_delivery.common
import dhl_delivery.dhl_process

import xmltodict
import datetime
import random


class Pickup:

    def __init__(self):
        self.date_now = datetime.datetime.now()

    def schedule_pickup(self, dict_param):
        return_status = True
        return_message = ''
        pickup_confirmation_number = ''
        return_dict = dict()
        common_obj = dhl_delivery.common.Common()

        # get the prepared XML data as string - this portion also check if xml wel formated  -  ie user sending correct data
        xml_formated_data = self.pickup_xml(dict_param)
        if xml_formated_data['status']:
            xml_formated_data_string = xml_formated_data['data']
        else:
            return_dict.update(
                {
                    'status': False,
                    'message': xml_formated_data['message'],
                }
            )
            return return_dict

        try:
            url_response_data = dict()
            dhl_process_obj = dhl_delivery.dhl_process.DhlProcess()
            returned_data = dhl_process_obj.process_pickup(
                xml_formated_data_string)
            if 'return_dhl_api_response' in returned_data.keys():
                return_status = True
                url_response_data = returned_data['return_dhl_api_response']
            else:
                return_status = False
                return_message = dhl_delivery.config.message_max_response_time
                return_dict.update(
                    {
                        'status': return_status,
                        'message': return_message,
                    }
                )
                return return_dict
        except:
            return_status = False
            return_message = dhl_delivery.config.message_dhl_url_down
            return_dict.update(
                {
                    'status': return_status,
                    'message': return_message,
                }
            )
        else:
            # format the data to dictionary and return
            dict_response = xmltodict.parse(url_response_data)

            # get the action note returned by DHL and check if it is  Success - if success then all are fine proceed
            try:
                action_note = common_obj.in_dictionary_all(
                    'ActionNote', dict_response)
                try:
                    if action_note == False:
                        action_note = list()
                except Exception as e:
                    action_note = list()
            except Exception as e:
                action_note = list()

            if 'Success' in action_note:  # action_note is a list type
                # get the confirmation number
                pickup_confirmation_number = common_obj.in_dictionary_all(
                    'ConfirmationNumber', dict_response)
                pickup_confirmation_number = ''.join(
                    map(str, pickup_confirmation_number))
                return_status = True
                return_message = dhl_delivery.config.message_pickup_success
            else:
                # some error by dhl which stopping to proceed
                error_found = common_obj.in_dictionary_all(
                    'ConditionData', dict_response)
                if error_found != False:
                    # some error by dhl
                    return_status = False
                    return_message = error_found[0]

            if dhl_delivery.dhl_response_flag:
                return_dict.update({'dhl_response': dict_response})
            if dhl_delivery.dhl_xml_flag:
                return_dict.update(
                    {'request_xml_to_dhl': xml_formated_data_string})

            return_dict.update(
                {
                    'status': return_status,
                    'message': return_message,
                    'pickup_confirmation_number': str(pickup_confirmation_number),
                }
            )
        return return_dict

    def pickup_xml(self, dict_param):
        try:
            common_obj = dhl_delivery.common.Common()
            addresses = dict_param['addresses']  # From and To Address
            pieces = dict_param['pieces']  # Measurement Units
            package = dict_param['package']  # delivery package
            # pickup time and schedule
            pickup_details = dict_param['pickup_details']

            optional_data = dict_param['optional_data']  # Measurement Units

            # as per DHL its must be between 28 to 32 char
            message_reference = ''.join(
                random.choice('0123456789') for i in range(28))
            # pickup_date = datetime.datetime.now() # changable

            # piece info
            piece_details_info = '<Pieces>' + str(len(pieces)) + '</Pieces>'
            piece_details_info += '<weight>'
            piece_details_info += '<Weight>' + \
                package['total_weight'] + '</Weight>'
            piece_details_info += '<WeightUnit>' + \
                package['weight_unit'] + '</WeightUnit>'
            piece_details_info += '</weight>'

            # reagon code based on country
            region_code = common_obj.get_dhl_region_code(
                addresses['from_country'])  # AP|AM|EU
            x_region_code_str = ''
            xsd_region_code_str = ''
            phone_extn_str = ''
            requestor_company_name = ''
            # place_details_str = '<DivisionName>' + addresses['from_state'] + '</DivisionName>'  # State of the pickup place - Optional - Max 35 char
            place_details_str = '<CountryCode>' + \
                addresses['from_country'] + '</CountryCode>'
            place_details_str += '<PostalCode>' + \
                addresses['from_zipcode'] + '</PostalCode>'
            region_code = addresses['from_region_code']  # AP|AM|EA
            if region_code == 'AP':
                x_region_code_str = region_code
                xsd_region_code_str = ''
                phone_extn_str = '<PhoneExtension></PhoneExtension>'
            elif region_code == 'AM':
                x_region_code_str = ''
                xsd_region_code_str = ''
                phone_extn_str = '<PhoneExtension></PhoneExtension>'
            elif region_code == 'EA':
                x_region_code_str = region_code
                xsd_region_code_str = '_' + region_code
                phone_extn_str = ''
                piece_details_info = ''  # only for EA
                requestor_company_name = '<CompanyName>' + \
                    addresses['from_company_name'] + '</CompanyName>'
                place_details_str = '<CountryCode>' + \
                    addresses['from_country'] + '</CountryCode>'
                place_details_str += '<PostalCode>' + \
                    addresses['from_zipcode'] + '</PostalCode>'

            # prepare the xml
            xml_str = '<?xml version="1.0" encoding="UTF-8"?>'

            xml_str += '<req:BookPickupRequest' + x_region_code_str + \
                ' xmlns:req="http://www.dhl.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            xml_str += 'xsi:schemaLocation="http://www.dhl.com ship-val-req' + \
                xsd_region_code_str + '.xsd">'

            # Request Service Header
            xml_str += '<Request>'
            xml_str += '<ServiceHeader>'
            xml_str += '<MessageTime>' + \
                self.date_now.strftime(
                    "%Y-%m-%dT%I:%M:%S+00:00") + '</MessageTime>'
            xml_str += '<MessageReference>' + message_reference + '</MessageReference>'
            xml_str += '<SiteID>' + dhl_delivery.dhl_site_id + '</SiteID>'
            xml_str += '<Password>' + dhl_delivery.dhl_site_password + '</Password>'
            xml_str += '</ServiceHeader>'
            xml_str += '</Request>'
            # Request Service Header ENDS

            # Requestor -- ie the sender - ie from where package will be collected - ie the from address
            xml_str += '<Requestor>'
            # C (Credit Card) | D (DHL account).
            xml_str += '<AccountType>D</AccountType>'
            xml_str += '<AccountNumber>' + dhl_delivery.dhl_account_no + '</AccountNumber>'  # ??
            xml_str += '<RequestorContact>'
            xml_str += '<PersonName>' + \
                addresses['from_name'] + '</PersonName>'
            xml_str += '<Phone>' + addresses['from_phone_no'] + '</Phone>'
            xml_str += phone_extn_str
            xml_str += '</RequestorContact>'
            xml_str += requestor_company_name
            xml_str += '</Requestor>'

            # Place
            xml_str += '<Place>'
            # B (Business), R (Residence) C (Business/Residence)
            xml_str += '<LocationType>' + \
                addresses['from_location_type'] + '</LocationType>'
            xml_str += '<CompanyName>' + \
                addresses['from_company_name'] + '</CompanyName>'
            xml_str += '<Address1>' + \
                addresses['from_address_line_one'] + '</Address1>'
            xml_str += '<Address2>' + \
                addresses['from_address_line_two'] + '</Address2>'
            # Package Location in the pickup place. E.g. Front Desk
            xml_str += '<PackageLocation>' + \
                addresses['from_package_location'] + '</PackageLocation>'
            xml_str += '<City>' + addresses['from_city'] + '</City>'
            xml_str += place_details_str
            xml_str += '</Place>'
            # Billing ENDS

            # Pickup
            xml_str += '<Pickup>'
            xml_str += '<PickupDate>' + \
                pickup_details['pickup_date'] + \
                '</PickupDate>'  # YYYY-MM-DD format
            xml_str += '<ReadyByTime>' + \
                pickup_details['ready_by_time'] + \
                '</ReadyByTime>'  # hh:mm ie 14:35
            xml_str += '<CloseTime>' + \
                pickup_details['close_time'] + '</CloseTime>'  # hh:mm ie 15:35
            xml_str += piece_details_info
            xml_str += '</Pickup>'
            # Pickup ENDS

            # Pickup Contact
            xml_str += '<PickupContact>'
            xml_str += '<PersonName>' + \
                addresses['from_name'] + '</PersonName>'
            xml_str += '<Phone>' + addresses['from_phone_no'] + '</Phone>'
            xml_str += phone_extn_str
            xml_str += '</PickupContact>'
            # Pickup Contact ENDS

            xml_str += '</req:BookPickupRequest' + x_region_code_str + '>'

            return_dict = {
                'status': True,
                'message': '',
                'data': xml_str,
            }
            return return_dict
        except:
            return_dict = {
                'status': False,
                'message': dhl_delivery.config.message_request_data_bad_format,
                'data': '',
            }
            return return_dict



import dhl_delivery
import dhl_delivery.config
import dhl_delivery.common
import dhl_delivery.dhl_process

import xmltodict

import datetime
import random


class Quote:

    def __init__(self):
        self.date_now = datetime.datetime.now()
        pass

    def get_quote(self, dict_param):
        return_status = True
        return_message = ''
        return_dict = dict()

        # get the prepared XML data as string - this portion also check if xml wel formated  -  ie user sending correct data
        xml_formated_data = self.quote_xml(dict_param)
        if xml_formated_data['status']:
            xml_formated_data_string = xml_formated_data['data']
        else:
            return_dict.update(
                {
                    'status': False,
                    'message': xml_formated_data['message'],
                }
            )
            return return_dict

        try:
            # here prepare to call the dhl api through the process which will check max execution time
            url_response_data = dict()
            dhl_process_obj = dhl_delivery.dhl_process.DhlProcess()
            returned_data = dhl_process_obj.process_quote(
                xml_formated_data_string)
            if 'return_dhl_api_response' in returned_data.keys():
                return_status = True
                url_response_data = returned_data['return_dhl_api_response']
            else:
                return_status = False
                return_message = dhl_delivery.config.message_max_response_time
                return_dict.update(
                    {
                        'status': return_status,
                        'message': return_message,
                    }
                )
                return return_dict
        except:
            return_status = False
            return_message = dhl_delivery.config.message_dhl_url_down
            return_dict.update(
                {
                    'status': return_status,
                    'message': return_message,
                }
            )
        else:
            # format the data to dictionary and return
            dict_response = xmltodict.parse(url_response_data)

            # check if error returned by dhl
            common_obj = dhl_delivery.common.Common()
            error_found = common_obj.in_dictionary_all(
                'ConditionData', dict_response)
            if error_found != False:
                # some error by dhl
                return_status = False
                return_message = error_found[0]
            else:
                # no error
                return_status = True
                return_message = dhl_delivery.config.message_quote_success
                dimensional_weight = common_obj.in_dictionary_all(
                    'DimensionalWeight', dict_response)[0]
                currency_code = common_obj.in_dictionary_all(
                    'CurrencyCode', dict_response)[0]
                weight_charge = common_obj.in_dictionary_all(
                    'WeightCharge', dict_response)[0]
                delivery_charge = common_obj.in_dictionary_all(
                    'deliveryCharge', dict_response)[0]
                product_shortName = common_obj.in_dictionary_all(
                    'ProductShortName', dict_response)[0]
                local_product_name = common_obj.in_dictionary_all(
                    'LocalProductName', dict_response)[0]
                weight_unit = common_obj.in_dictionary_all(
                    'WeightUnit', dict_response)[0]

                return_dict.update({
                    'quote_data': {
                        'dimensional_weight': dimensional_weight,
                        'weight_unit': weight_unit,
                        'currency_code': currency_code,
                        # 'weight_charge': weight_charge,
                        'delivery_charge': delivery_charge,
                        'product_shortName': product_shortName,
                        'local_product_name': local_product_name,
                    }
                })

            if dhl_delivery.dhl_xml_flag:
                return_dict.update(
                    {'request_xml_to_dhl': xml_formated_data_string})
            if dhl_delivery.dhl_response_flag:
                return_dict.update({'dhl_response': dict_response})

            return_dict.update(
                {
                    'status': return_status,
                    'message': return_message,
                }
            )
        return return_dict

    def quote_xml(self, dict_param):
        try:
            addresses = dict_param['addresses']  # From and To Address
            units = dict_param['units']  # Measurement Units
            pieces = dict_param['pieces']  # Measurement Units
            optional_data = dict_param['optional_data']  # Measurement Units

            # as per DHL its must be between 28 to 32 char
            message_reference = ''.join(
                random.choice('0123456789') for i in range(28))
            # pickup_date = datetime.datetime.now() # changable

            xml_str = '<?xml version="1.0" encoding="UTF-8"?>'

            xml_str += '<p:DCTRequest xmlns:p="http://www.dhl.com" xmlns:p1="http://www.dhl.com/datatypes" xmlns:p2="http://www.dhl.com/DCTRequestdatatypes" '
            xml_str += 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.dhl.com DCT-req.xsd ">'

            # GetQuote
            xml_str += '<GetQuote>'

            # Request
            xml_str += '<Request>'
            xml_str += '<ServiceHeader>'
            xml_str += '<MessageTime>' + \
                self.date_now.strftime(
                    "%Y-%m-%dT%I:%M:%S+00:00") + '</MessageTime>'
            xml_str += '<MessageReference>' + message_reference + '</MessageReference>'
            xml_str += '<SiteID>' + dhl_delivery.dhl_site_id + '</SiteID>'
            xml_str += '<Password>' + dhl_delivery.dhl_site_password + '</Password>'
            xml_str += '</ServiceHeader>'
            xml_str += '</Request>'
            # Request ENDS

            # From Address
            xml_str += '<From>'
            xml_str += '<CountryCode>' + \
                addresses['from_country'] + '</CountryCode>'
            xml_str += '<Postalcode>' + \
                addresses['from_zipcode'] + '</Postalcode>'
            xml_str += '<City>' + addresses['from_city'] + '</City>'
            xml_str += '</From>'
            # From Address ENDS

            # BkgDetails
            try:
                if 'quote_for_date' in optional_data.keys():
                    quote_for_date = optional_data['quote_for_date']
                    quote_for_date = str(quote_for_date)
                else:
                    quote_for_date = self.date_now.strftime("%Y-%m-%d")
            except Exception as e:
                quote_for_date = self.date_now.strftime("%Y-%m-%d")

            xml_str += '<BkgDetails>'
            xml_str += '<PaymentCountryCode>MY</PaymentCountryCode>'
            xml_str += '<Date>' + quote_for_date + '</Date>'  # for weekend its change for us
            xml_str += '<ReadyTime>PT10H21M</ReadyTime>'
            xml_str += '<ReadyTimeGMTOffset>+01:00</ReadyTimeGMTOffset>'
            xml_str += '<DimensionUnit>' + \
                units['dimension_unit'] + '</DimensionUnit>'
            xml_str += '<WeightUnit>' + units['weight_unit'] + '</WeightUnit>'

            xml_str += '<Pieces>'
            piece_id = 1
            for piece in pieces:
                xml_str += '<Piece>'
                xml_str += '<PieceID>' + str(piece_id) + '</PieceID>'
                xml_str += '<Height>' + \
                    str(piece['piece_height']) + '</Height>'
                xml_str += '<Depth>' + str(piece['piece_depth']) + '</Depth>'
                xml_str += '<Width>' + str(piece['piece_width']) + '</Width>'
                xml_str += '<Weight>' + \
                    str(piece['piece_weight']) + '</Weight>'
                xml_str += '</Piece>'
                piece_id = piece_id + 1
            xml_str += '</Pieces>'

            xml_str += '<PaymentAccountNumber>' + \
                dhl_delivery.dhl_account_no + '</PaymentAccountNumber>'
            xml_str += '<IsDutiable>' + \
                optional_data['is_dutiable'] + '</IsDutiable>'
            xml_str += '<NetworkTypeCode>AL</NetworkTypeCode>'  # decide

            xml_str += '<QtdShp>'
            xml_str += '<GlobalProductCode>P</GlobalProductCode>'
            # xml_str += '<LocalProductCode>P</LocalProductCode>'  # taken away as for GB to other countries it will not work - confirmed with dhl
            xml_str += '</QtdShp>'

            ins_val = optional_data.get('insured_value', 0)
            ins_ccy = optional_data.get('insured_currency', '')
            if ins_val:
                """ as per dhl new instruction we should pass it if we want qoute with insurance.
                The new change on dhl side is that if it even passsed as 0 still it will return quote with insurance.
                therefore no choice but to remove it if insurance value is 0 or empty to get quote without insurance.
                This change made on 14.11.2017
                """
                xml_str += '<InsuredValue>' + str(ins_val) + '</InsuredValue>'
                xml_str += '<InsuredCurrency>' + ins_ccy + '</InsuredCurrency>'
            xml_str += '</BkgDetails>'
            # BkgDetails ENDS

            # To Address
            xml_str += '<To>'
            xml_str += '<CountryCode>' + \
                addresses['to_country'] + '</CountryCode>'
            xml_str += '<Postalcode>' + \
                addresses['to_zipcode'] + '</Postalcode>'
            xml_str += '<City>' + addresses['to_city'] + '</City>'
            xml_str += '</To>'
            # To Address ENDS

            xml_str += '<Dutiable>'
            xml_str += '<DeclaredCurrency>' + \
                optional_data['declared_currency'] + \
                '</DeclaredCurrency>'  # decide
            xml_str += '<DeclaredValue>' + \
                optional_data['declared_value'] + '</DeclaredValue>'
            xml_str += '</Dutiable>'

            xml_str += '</GetQuote>'
            # GetQuote ENDS

            xml_str += '</p:DCTRequest>'

            return_dict = {
                'status': True,
                'message': '',
                'data': xml_str,
            }
            return return_dict
        except:
            return_dict = {
                'status': False,
                'message': dhl_delivery.config.message_request_data_bad_format,
                'data': '',
            }
            return return_dict



import time
from datetime import datetime
import multiprocessing

import dhl_delivery
import dhl_delivery.config
from urllib import request
from urllib.request import urlopen


class DhlProcess:
    def __init__(self):
        pass

    @staticmethod
    def check_utilities(n, return_dict):
        return_dict['return_check_utilities'] = True

    @staticmethod
    def stop_process(p, start_time, time_to_stop_in_second):
        start_this = time.time()
        while time.time() < start_this + time_to_stop_in_second:
            time_end_this = datetime.now()
            delta = time_end_this - start_time
            time_interval_in_second = delta.total_seconds()

            if time_interval_in_second >= time_to_stop_in_second:
                try:
                    p.terminate()
                except:
                    pass

    def process_quote(self, xml_formated_data_string):
        time_now = datetime.now()

        manager = multiprocessing.Manager()
        return_dict = manager.dict()
        jobs = []
        p_one = multiprocessing.Process(
            target=self.check_utilities, name="name_check_utilities", args=(20, return_dict))
        jobs.append(p_one)
        p_two = multiprocessing.Process(target=self.call_dhl_quote_api, name="name_call_dhl_quote_api", args=(
            xml_formated_data_string, return_dict))
        jobs.append(p_two)
        p_stop = multiprocessing.Process(target=self.stop_process, name="name_stop_process", args=(
            p_two, time_now, dhl_delivery.max_response_time))

        # Start the jobs
        p_one.start()
        p_two.start()
        p_stop.start()

        for job in jobs:
            job.join()

        return return_dict

    @staticmethod
    def call_dhl_quote_api(xml_formated_data_string, return_dict):
        url = dhl_delivery.config.dhl_api_url
        # url_request = urllib2.Request(url, xml_formated_data_string)
        url_response = urlopen(url, xml_formated_data_string)
        # getting as XML String from DHL Response
        url_response_data = url_response.read()
        return_dict['return_dhl_api_response'] = url_response_data

    def process_delivery(self, xml_formated_data_string):
        time_now = datetime.now()

        manager = multiprocessing.Manager()
        return_dict = manager.dict()
        jobs = []
        p_one = multiprocessing.Process(
            target=self.check_utilities, name="name_check_utilities", args=(20, return_dict))
        jobs.append(p_one)
        p_two = multiprocessing.Process(target=self.call_dhl_delivery_api, name="name_call_dhl_delivery_api", args=(
            xml_formated_data_string, return_dict))
        jobs.append(p_two)
        p_stop = multiprocessing.Process(target=self.stop_process, name="name_stop_process", args=(
            p_two, time_now, dhl_delivery.max_response_time))

        # Start the jobs
        p_one.start()
        p_two.start()
        p_stop.start()

        for job in jobs:
            job.join()

        return return_dict

    @staticmethod
    def call_dhl_delivery_api(xml_formated_data_string, return_dict):
        url = dhl_delivery.config.dhl_api_url
        # url_request = urllib2.Request(url, xml_formated_data_string)
        url_response = urlopen(url, xml_formated_data_string)
        # getting as XML String from DHL Response
        url_response_data = url_response.read()
        return_dict['return_dhl_api_response'] = url_response_data

    def process_pickup(self, xml_formated_data_string):
        time_now = datetime.now()

        manager = multiprocessing.Manager()
        return_dict = manager.dict()
        jobs = []
        p_one = multiprocessing.Process(
            target=self.check_utilities, name="name_check_utilities", args=(20, return_dict))
        jobs.append(p_one)
        p_two = multiprocessing.Process(target=self.call_dhl_pickup_api, name="name_call_dhl_pickup_api", args=(
            xml_formated_data_string, return_dict))
        jobs.append(p_two)
        p_stop = multiprocessing.Process(target=self.stop_process, name="name_stop_process", args=(
            p_two, time_now, dhl_delivery.max_response_time))

        # Start the jobs
        p_one.start()
        p_two.start()
        p_stop.start()

        for job in jobs:
            job.join()

        return return_dict

    @staticmethod
    def call_dhl_pickup_api(xml_formated_data_string, return_dict):
        url = dhl_delivery.config.dhl_api_url
        # url_request = urllib2.Request(url, xml_formated_data_string)
        url_response = urlopen(url, xml_formated_data_string)
        # getting as XML String from DHL Response
        url_response_data = url_response.read()
        return_dict['return_dhl_api_response'] = url_response_data



class Common:
    def __init__(self):
        pass

    def in_dictionary(self, field, dict_data):
        """ Search for string by key in a dictionary.
            param 1: the key as string
            param 2: the dictionary data
            returns: if found then returns the value of the key as string else returns False
        """
        found_flag = False
        eureka = ''
        if field in dict_data:
            found_flag = True
            eureka = dict_data[field]
            # return dict_data[field]
        for k, v in dict_data.items():
            if isinstance(v, dict):
                eureka = self.in_dictionary(field, v)
                if eureka is not None:
                    found_flag = True
                    # return eureka
        if found_flag == True:
            return eureka
        else:
            return False

    def in_dictionary_all(self, field, dict_data, not_found_return_str=''):
        """ Search for string by key in a dictionary.
            param 1: the key as string
            param 2: the  dictionary data
            returns: if found then returns the all occurrence-value of the key as a list else returns False or the specifed string
        """
        found_flag = False
        fields_found = []

        for key, value in dict_data.items():

            if key == field:
                found_flag = True
                fields_found.append(value)

            elif isinstance(value, dict):
                results = self.in_dictionary_all(field, value)
                if type(results) == list:
                    for result in results:
                        found_flag = True
                        fields_found.append(result)

            elif isinstance(value, list):
                for item in value:
                    if isinstance(item, dict):
                        more_results = self.in_dictionary_all(field, item)
                        if type(more_results) == list:
                            for another_result in more_results:
                                found_flag = True
                                fields_found.append(another_result)

        if found_flag == True:
            return fields_found
        else:
            if not_found_return_str:
                return not_found_return_str
            else:
                return False

    def get_dhl_region_code(self, country_code):
        """
        Get region code AP|AM|EU by country code
        """
        region_code = ''
        ap_list = ['AE', 'AF', 'AL', 'AM', 'AO', 'AU', 'BA', 'BD', 'BH', 'BN',
                   'BY', 'CI', 'CM', 'CN', 'CY', 'DJ', 'DZ', 'EG', 'ET', 'FJ', 'GA', 'GH',
                   'HK', 'HR', 'ID', 'IL', 'IN', 'IR', 'JO', 'JP', 'KE', 'KR', 'KW', 'KZ',
                   'LA', 'LB', 'LK', 'MA', 'MD', 'MG', 'MK', 'MO', 'MT', 'MU', 'MY', 'NA',
                   'NG', 'NZ', 'OM', 'PH', 'PK', 'QA', 'RE', 'RS', 'RU', 'SA', 'SD', 'SG',
                   'SN', 'TH', 'TR', 'TW', 'TZ', 'UA', 'UG', 'UZ', 'VN', 'YE', 'ZA']

        eu_list = ['AT', 'BE', 'BG', 'CH', 'CZ', 'DE', 'DK', 'EE', 'ES',
                   'FI', 'FR', 'GB', 'GR', 'HU', 'IE', 'IS', 'IT', 'LT', 'LU', 'LV',
                   'NL', 'NO', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK']

        am_list = ['AI', 'AR', 'AW', 'BB', 'BM', 'BO', 'BR', 'BS', 'CA', 'CL',
                   'CO', 'CR', 'DM', 'DO', 'EC', 'GP', 'GT', 'GU', 'GY', 'HN', 'HT', 'JM',
                   'KN', 'KY', 'LC', 'MQ', 'MX', 'NI', 'PA', 'PE', 'PR', 'PY', 'SV', 'TC',
                   'TT', 'US', 'UY', 'VE', 'VG', 'VI', 'XM', 'XY']

        if country_code in ap_list:
            region_code = 'AP'
        elif country_code in eu_list:
            region_code = 'EU'
        elif country_code in am_list:
            region_code = 'AM'
        return region_code



dhl_api_url = 'https://xmlpi-ea.dhl.com/XMLdeliveryServlet'
message_dhl_url_down = 'DHL API is not accessible at this moment. Please try after few minutes.'
message_max_response_time = 'Maximum execution time to call DHL API passed. DHL API not able to response within given time.'
message_quote_success = 'DHL quotes returned.'
message_delivery_success = 'DHL delivery created.'
message_pickup_success = 'DHL Pickup created.'

message_request_data_bad_format = 'The data you have sent is not well formated. It looks like some key-value pair are missing. Please check the documentation and send proper request data.'



#from distutils.core import setup
from setuptools import setup

setup(
    name='dhl_delivery',
    version='2.0.10',
    author='Iván Salazar',
    author_email='ivangio.salazar@gmail.com',
    packages=['dhl_delivery'],
    url='http://pypi.python.org/pypi/dhl_delivery/',
    license='MIT',
    description='DHL Capability - Quote & Shipment',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    install_requires=[
        'xmltodict',
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)



import dhl_delivery
import dhl_delivery.config
import dhl_delivery.common
import dhl_delivery.dhl_process

import xmltodict

import datetime
import random
import base64
import os
import errno


class Shipment:

    def __init__(self):
        self.date_now = datetime.datetime.now()

    def get_shipment(self, dict_param):
        return_status = True
        return_message = ''
        pickup_return_status = False
        pickup_return_message = ''
        airway_bill_number = ''
        output_pdf_image_str = ''  # base 64 encoded pdf
        awb_pdf_file_path_name = ''
        awb_pdf_file_name = ''
        return_dict = dict()
        pickup_returned_data = dict()
        pickup_confirmation_number = ''
        product_short_name = ''
        package_type = ''
        chargeable_weight = ''
        dimensional_weight = ''
        insured_amount = ''
        shipment_date = ''  # ie pickup or dropoff to dhl date, not the date when its being created

        request_for_pickup = False
        common_obj = dhl_delivery.common.Common()

        # get the prepared XML data as string - this portion also check if xml wel formated  -  ie user sending correct data
        xml_formated_data = self.shipment_xml(dict_param)
        if xml_formated_data['status']:
            xml_formated_data_string = xml_formated_data['data']
        else:
            return_dict.update(
                {
                    'status': False,
                    'message': xml_formated_data['message'],
                }
            )
            return return_dict

        try:
            url_response_data = dict()
            dhl_process_obj = dhl_delivery.dhl_process.DhlProcess()
            returned_data = dhl_process_obj.process_shipment(
                xml_formated_data_string)
            if 'return_dhl_api_response' in returned_data.keys():
                return_status = True
                url_response_data = returned_data['return_dhl_api_response']
            else:
                return_status = False
                return_message = dhl_delivery.config.message_max_response_time
                return_dict.update(
                    {
                        'status': return_status,
                        'message': return_message,
                    }
                )
                return return_dict
        except:
            return_status = False
            return_message = dhl_delivery.config.message_dhl_url_down
            return_dict.update(
                {
                    'status': return_status,
                    'message': return_message,
                }
            )
        else:
            # format the data to dictionary and return
            dict_response = xmltodict.parse(url_response_data)

            # get the action note returned by DHL and check if it is  Success - if success then all are fine proceed
            action_note = common_obj.in_dictionary_all(
                'ActionNote', dict_response, 'nothing found')
            if 'Success' in action_note:  # action_note is a list type
                return_status = True
                return_message = dhl_delivery.config.message_shipment_success
                # set pick up - if pickup provided
                if 'pickup_details' in dict_param:
                    if len(dict_param['pickup_details']) > 0:
                        request_for_pickup = True
                        pickup_returned_data = dhl_delivery.pickup.schedule_pickup(
                            dict_param)
                        if pickup_returned_data['status']:
                            pickup_confirmation_number_temp = pickup_returned_data[
                                'pickup_confirmation_number']
                            pickup_confirmation_number = ''.join(
                                map(str, pickup_confirmation_number_temp))
                        pickup_return_status = pickup_returned_data['status']
                        pickup_return_message = pickup_returned_data['message']

                # get the airway bill number
                airway_bill_number = common_obj.in_dictionary_all(
                    'AirwayBillNumber', dict_response)
                airway_bill_number = ''.join(map(str, airway_bill_number))

                # get product short name
                product_short_name = common_obj.in_dictionary_all(
                    'ProductShortName', dict_response)
                product_short_name = ''.join(map(str, product_short_name))

                # get package type
                package_type = common_obj.in_dictionary_all(
                    'PackageType', dict_response)
                package_type = ''.join(map(str, package_type))

                # Chargeable Weight
                chargeable_weight = common_obj.in_dictionary_all(
                    'ChargeableWeight', dict_response)
                chargeable_weight = ''.join(map(str, chargeable_weight))

                # Dimensional Weight
                dimensional_weight = common_obj.in_dictionary_all(
                    'DimensionalWeight', dict_response)
                dimensional_weight = ''.join(map(str, dimensional_weight))

                # Insured Amount
                insured_amount = common_obj.in_dictionary_all(
                    'InsuredAmount', dict_response)
                insured_amount = ''.join(map(str, insured_amount))

                # Shipment Date ie pickup or dropoff to dhl, not the date when its being created
                shipment_date = common_obj.in_dictionary_all(
                    'ShipmentDate', dict_response)
                shipment_date = ''.join(map(str, shipment_date))

                if 'optional_data' in dict_param.keys():
                    optional_data = dict_param['optional_data']

                    # get the pdf file image pdf (base 64 encoded)
                    if 'return_pdf_image_str' in optional_data.keys():
                        if optional_data['return_pdf_image_str']:
                            output_image_list = common_obj.in_dictionary_all(
                                'OutputImage', dict_response)
                            output_pdf_image_str = ''.join(
                                map(str, output_image_list))

                    # now create the pdf file form the image (base 64 string for pdf)
                    if 'return_pdf_file' in optional_data.keys():
                        if optional_data['return_pdf_file']:
                            create_pdf_data = self.create_pdf_awb(
                                dict_param, dict_response, airway_bill_number)
                            awb_pdf_file_path_name = create_pdf_data['awb_pdf_file_path_name']
                            awb_pdf_file_name = create_pdf_data['awb_pdf_file_name']
            else:
                # some error by dhl which stopping to proceed
                error_found = common_obj.in_dictionary_all(
                    'ConditionData', dict_response)
                if error_found != False:
                    # some error by dhl
                    return_status = False
                    return_message = error_found[0]

            if dhl_delivery.dhl_response_flag:
                return_dict.update(
                    {'dhl_response': {'dhl_shipment_response': dict_response}})
                if 'dhl_response' in pickup_returned_data.keys():
                    return_dict.update({'dhl_response': {'dhl_shipment_response': dict_response,
                                                         'dhl_pickup_response': pickup_returned_data['dhl_response']}})
            if dhl_delivery.dhl_xml_flag:
                return_dict.update(
                    {'request_xml_to_dhl': xml_formated_data_string})

            return_dict.update(
                {
                    'shipment_response': {
                        'status': return_status,
                        'message': return_message,
                        'airway_bill_number': airway_bill_number,
                        'awb_pdf_file_path_name': awb_pdf_file_path_name,
                        'awb_pdf_file_name': awb_pdf_file_name,
                        'output_pdf_image_str': output_pdf_image_str,
                        'product_short_name': product_short_name,
                        'package_type': package_type,
                        'chargeable_weight': chargeable_weight,
                        'dimensional_weight': dimensional_weight,
                        'insured_amount': insured_amount,
                        'shipment_date': shipment_date
                    },
                    'pickup_response': {
                        'status': pickup_return_status,
                        'message': pickup_return_message,
                        # True if user set up request for pickup else False
                        'request_for_pickup': request_for_pickup,
                        'pickup_confirmation_number': pickup_confirmation_number
                    }
                }
            )
        return return_dict

    def create_pdf_awb(self, dict_param, dict_response, airway_bill_number=''):
        common_obj = dhl_delivery.common.Common()
        awb_pdf_file_path_name = ''
        # now create the pdf file form the image (base 64 string for pdf)
        if 'optional_data' in dict_param.keys():
            awb_pdf_file_path = ''
            awb_pdf_file_name = ''
            # awb file name and path to store
            optional_data = dict_param['optional_data']
            if 'awb_pdf_file_path' in optional_data.keys():
                awb_pdf_file_path = optional_data['awb_pdf_file_path']
            if not awb_pdf_file_path:
                awb_pdf_file_path = os.path.join('pdf_shipment_label_default')
            if awb_pdf_file_path:
                # file path and name
                if 'awb_pdf_file_name' in optional_data.keys():
                    awb_pdf_file_name = optional_data['awb_pdf_file_name']
                if not awb_pdf_file_name:
                    awb_pdf_file_name = 'dhl_awb.pdf'
                # Change the name of awb - prefix the file name with Airway Bill Number
                if not airway_bill_number:
                    airway_bill_number = common_obj.in_dictionary_all(
                        'AirwayBillNumber', dict_response)
                    airway_bill_number = ''.join(map(str, airway_bill_number))
                awb_pdf_file_name = airway_bill_number + '__' + awb_pdf_file_name

                # create the pdf file from the base 64
                output_image_list = common_obj.in_dictionary_all(
                    'OutputImage', dict_response)
                output_image_str = ''.join(map(str, output_image_list))
                file_data = base64.urlsafe_b64decode(
                    output_image_str.encode('UTF-8'))

                # if folder not found then create one
                awb_pdf_file_path_name = os.path.join(
                    awb_pdf_file_path, awb_pdf_file_name)
                if not os.path.exists(os.path.dirname(awb_pdf_file_path_name)):
                    try:
                        os.makedirs(os.path.dirname(awb_pdf_file_path_name))
                    except OSError as exc:
                        if exc.errno != errno.EEXIST:
                            raise
                with open(awb_pdf_file_path_name, 'w') as the_file:  # wb dont save
                    the_file.write(file_data)
                    the_file.close()
        return {'awb_pdf_file_path_name': awb_pdf_file_path_name, 'awb_pdf_file_name': awb_pdf_file_name}

    def shipment_xml(self, dict_param):
        try:
            addresses = dict_param['addresses']  # From and To Address
            pieces = dict_param['pieces']  # Measurement Units
            package = dict_param['package']  # shipment package

            # optional_data = dict_param['optional_data']  # Measurement Units

            # as per DHL its must be between 28 to 32 char
            message_reference = ''.join(
                random.choice('0123456789') for i in range(28))
            # pickup_date = datetime.datetime.now() # changable

            xml_str = '<?xml version="1.0" encoding="UTF-8"?>'

            xml_str += '<req:ShipmentValidateRequestAP xmlns:req="http://www.dhl.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            xml_str += 'xsi:schemaLocation="http://www.dhl.com ship-val-req_AP.xsd">'

            # Request Service Header
            xml_str += '<Request>'
            xml_str += '<ServiceHeader>'
            xml_str += '<MessageTime>' + \
                self.date_now.strftime(
                    "%Y-%m-%dT%I:%M:%S+00:00") + '</MessageTime>'
            xml_str += '<MessageReference>' + message_reference + '</MessageReference>'
            xml_str += '<SiteID>' + dhl_delivery.dhl_site_id + '</SiteID>'
            xml_str += '<Password>' + dhl_delivery.dhl_site_password + '</Password>'
            xml_str += '</ServiceHeader>'
            xml_str += '</Request>'
            # Request Service Header ENDS

            xml_str += '<LanguageCode>en</LanguageCode>'
            xml_str += '<PiecesEnabled>Y</PiecesEnabled>'

            # Billing
            xml_str += '<Billing>'
            xml_str += '<ShipperAccountNumber>' + \
                dhl_delivery.dhl_account_no + '</ShipperAccountNumber>'
            xml_str += '<ShippingPaymentType>' + \
                dhl_delivery.shipping_payment_type + '</ShippingPaymentType>'
            xml_str += '<DutyPaymentType>' + \
                dhl_delivery.duty_payment_type + '</DutyPaymentType>'
            xml_str += '</Billing>'
            # Billing ENDS

            # Consignee - ie the receiver - is to where the package will be send to - ie the to address
            xml_str += '<Consignee>'
            xml_str += '<CompanyName>' + \
                addresses['to_company_name'] + '</CompanyName>'
            xml_str += '<AddressLine>' + \
                addresses['to_address_line_one'] + '</AddressLine>'
            xml_str += '<AddressLine>' + \
                addresses['to_address_line_two'] + '</AddressLine>'  # optionl
            xml_str += '<City>' + addresses['to_city'] + '</City>'
            xml_str += '<PostalCode>' + \
                addresses['to_zipcode'] + '</PostalCode>'
            xml_str += '<CountryCode>' + \
                addresses['to_country'] + '</CountryCode>'
            xml_str += '<CountryName>' + \
                addresses['to_country_name'] + '</CountryName>'
            xml_str += '<Contact>'
            xml_str += '<PersonName>' + addresses['to_name'] + '</PersonName>'
            xml_str += '<PhoneNumber>' + \
                addresses['to_phone_no'] + '</PhoneNumber>'
            xml_str += '</Contact>'
            xml_str += '</Consignee>'
            # Consignee ENDS

            # Dutiable
            xml_str += '<Dutiable>'
            xml_str += '<DeclaredValue>' + \
                str(package['declared_value']) + '</DeclaredValue>'
            # currency code  -  3 letter abbriviation
            xml_str += '<DeclaredCurrency>' + \
                package['declared_currency'] + '</DeclaredCurrency>'
            xml_str += '</Dutiable>'
            # Dutiable ENDS

            # Reference
            xml_str += '<Reference>'
            xml_str += '<ReferenceID>' + \
                str(package['reference_id']) + '</ReferenceID>'
            xml_str += '<ReferenceType>St</ReferenceType>'  # optional
            xml_str += '</Reference>'
            # Reference ENDS

            # ShipmentDetails
            xml_str += '<ShipmentDetails>'
            xml_str += '<NumberOfPieces>' + \
                str(len(pieces)) + '</NumberOfPieces>'
            # currency code  -  3 letter abbriviation
            xml_str += '<CurrencyCode>' + \
                package['declared_currency'] + '</CurrencyCode>'
            xml_str += '<Pieces>'
            piece_id = 1
            for piece in pieces:
                xml_str += '<Piece>'
                xml_str += '<PieceID>' + str(piece_id) + '</PieceID>'
                xml_str += '<PackageType>' + \
                    str(piece['package_type']) + '</PackageType>'  # DF|YP etc
                xml_str += '<Weight>' + \
                    str(piece['piece_weight']) + '</Weight>'
                xml_str += '<Depth>' + str(piece['piece_depth']) + '</Depth>'
                xml_str += '<Width>' + str(piece['piece_width']) + '</Width>'
                xml_str += '<Height>' + \
                    str(piece['piece_height']) + '</Height>'
                xml_str += '</Piece>'
                piece_id = piece_id + 1
            xml_str += '</Pieces>'
            xml_str += '<PackageType>' + \
                str(package['package_type']) + '</PackageType>'  # DF|YP etc
            xml_str += '<Weight>' + str(package['total_weight']) + '</Weight>'
            xml_str += '<DimensionUnit>' + \
                str(package['dimension_unit']) + '</DimensionUnit>'
            xml_str += '<WeightUnit>' + \
                str(package['weight_unit']) + '</WeightUnit>'
            xml_str += '<GlobalProductCode>' + \
                str(package['global_product_code']) + \
                '</GlobalProductCode>'  # as P
            xml_str += '<LocalProductCode>' + \
                str(package['local_product_code']) + \
                '</LocalProductCode>'  # as P
            xml_str += '<DoorTo>DD</DoorTo>'
            # Shipment date for when package(s) will be shipped (but usually current date)  ' + self.date_now.strftime("%Y-%m-%d") + '
            xml_str += '<Date>' + str(package['shipment_date']) + '</Date>'
            xml_str += '<Contents>' + \
                str(package['content_description']) + '</Contents>'
            xml_str += '<IsDutiable>' + \
                str(package['is_dutiable']) + \
                '</IsDutiable>'  # Yes|No - Default is No
            # Insured Amount (Required if Special Service of ‘I’ – Insurance is provided in request)
            xml_str += '<InsuredAmount>' + \
                str(package['insured_amount']) + '</InsuredAmount>'
            xml_str += '</ShipmentDetails>'
            # ShipmentDetails ENDS

            # Shipper - is the sender - ie from where package will be collected - ie the from address
            xml_str += '<Shipper>'
            xml_str += '<ShipperID>AXW56546</ShipperID>'  # a random number <=30
            xml_str += '<CompanyName>' + \
                addresses['from_company_name'] + '</CompanyName>'
            xml_str += '<AddressLine>' + \
                addresses['from_address_line_one'] + '</AddressLine>'
            xml_str += '<AddressLine>' + \
                addresses['from_address_line_two'] + \
                '</AddressLine>'  # optional
            xml_str += '<City>' + addresses['from_city'] + '</City>'
            xml_str += '<PostalCode>' + \
                addresses['from_zipcode'] + '</PostalCode>'
            xml_str += '<CountryCode>' + \
                addresses['from_country'] + '</CountryCode>'
            xml_str += '<CountryName>' + \
                addresses['from_country_name'] + '</CountryName>'
            xml_str += '<Contact>'
            xml_str += '<PersonName>' + \
                addresses['from_name'] + '</PersonName>'
            xml_str += '<PhoneNumber>' + \
                addresses['from_phone_no'] + '</PhoneNumber>'
            xml_str += '</Contact>'
            xml_str += '</Shipper>'
            # Shipper ENDS

            # Special Service - Totally optional
            if 'special_service_type' in package.keys():
                if package['special_service_type']:
                    xml_str += '<SpecialService>'
                    # Insured Amount (Required if Special Service of ‘I’ – Insurance is provided in request)
                    xml_str += '<SpecialServiceType>' + \
                        str(package['special_service_type']) + \
                        '</SpecialServiceType>'
                    xml_str += '</SpecialService>'
            # Special Service

            xml_str += '<LabelImageFormat>PDF</LabelImageFormat>'
            xml_str += '</req:ShipmentValidateRequestAP>'

            return_dict = {
                'status': True,
                'message': '',
                'data': xml_str,
            }
            return return_dict
        except:
            return_dict = {
                'status': False,
                'message': dhl_delivery.config.message_request_data_bad_format,
                'data': '',
            }
            return return_dict
